from collections import defaultdict

__all__ = ["to_mass", "count_elements", "calc_molecularweights"]


atom2mass = {
    "H": 1.00797,
    "He": 4.0026,
    "Li": 6.941,
    "Be": 9.01218,
    "B": 10.81,
    "C": 12.011,
    "N": 14.0067,
    "O": 15.9994,
    "F": 18.998403,
    "Ne": 20.179,
    "Na": 22.98977,
    "Mg": 24.305,
    "Al": 26.98154,
    "Si": 28.0855,
    "P": 30.97376,
    "S": 32.06,
    "Cl": 35.453,
    "K": 39.0983,
    "Ar": 39.948,
    "Ca": 40.08,
    "Sc": 44.9559,
    "Ti": 47.9,
    "V": 50.9415,
    "Cr": 51.996,
    "Mn": 54.938,
    "Fe": 55.847,
    "Ni": 58.7,
    "Co": 58.9332,
    "Cu": 63.546,
    "Zn": 65.38,
    "Ga": 69.72,
    "Ge": 72.59,
    "As": 74.9216,
    "Se": 78.96,
    "Br": 79.904,
    "Kr": 83.8,
    "Rb": 85.4678,
    "Sr": 87.62,
    "Y": 88.9059,
    "Zr": 91.22,
    "Nb": 92.9064,
    "Mo": 95.94,
    "Tc": 98.0,
    "Ru": 101.07,
    "Rh": 102.9055,
    "Pd": 106.4,
    "Ag": 107.868,
    "Cd": 112.41,
    "In": 114.82,
    "Sn": 118.69,
    "Sb": 121.75,
    "I": 126.9045,
    "Te": 127.6,
    "Xe": 131.3,
    "Cs": 132.9054,
    "Ba": 137.33,
    "La": 138.9055,
    "Ce": 140.12,
    "Pr": 140.9077,
    "Nd": 144.24,
    "Pm": 145.0,
    "Sm": 150.4,
    "Eu": 151.96,
    "Gd": 157.25,
    "Tb": 158.9254,
    "Dy": 162.5,
    "Ho": 164.9304,
    "Er": 167.26,
    "Tm": 168.9342,
    "Yb": 173.04,
    "Lu": 174.967,
    "Hf": 178.49,
    "Ta": 180.9479,
    "W": 183.85,
    "Re": 186.207,
    "Os": 190.2,
    "Ir": 192.22,
    "Pt": 195.09,
    "Au": 196.9665,
    "Hg": 200.59,
    "Tl": 204.37,
    "Pb": 207.2,
    "Bi": 208.9804,
    "Po": 209.0,
    "At": 210.0,
    "Rn": 222.0,
    "Fr": 223.0,
    "Ra": 226.0254,
    "Ac": 227.0278,
    "Pa": 231.0359,
    "Th": 232.0381,
    "Np": 237.0482,
    "U": 238.029,
    "Pu": 242.0,
    "Am": 243.0,
    "Bk": 247.0,
    "Cm": 247.0,
    "No": 250.0,
    "Cf": 251.0,
    "Es": 252.0,
    "Hs": 255.0,
    "Mt": 256.0,
    "Fm": 257.0,
    "Md": 258.0,
    "Lr": 260.0,
    "Rf": 261.0,
    "Bh": 262.0,
    "Db": 262.0,
    "Sg": 263.0,
    "Uun": 269.0,
    "Uuu": 272.0,
    "Uub": 277.0,
}


def to_mass(element: str):
    assert element in atom2mass, f"{element} is not included."
    return atom2mass[element]


def count_elements(elements: str) -> dict[str, int]:
    element_dict = defaultdict(int)
    tmp_element = ""
    for char in elements:
        if char.isupper():
            if tmp_element:
                element_dict[tmp_element] += 1
                tmp_element = ""
        elif char.isnumeric():
            element_dict[tmp_element] += int(char)
            tmp_element = ""
            continue
        tmp_element += char
    if tmp_element:
        element_dict[tmp_element] += 1
    return element_dict


def calc_molecularweights(element_count: dict[str, int]):
    mw = 0.0
    for element, num in element_count.items():
        mw += to_mass(element=element) * num
    return mw
